﻿

namespace NRuler.Test.Common
{
    using System;
    using System.IO;
    using System.Runtime.Serialization.Formatters.Binary;
    using NRuler.Common;
    using NRuler.Conditions;
    using NRuler.Rete;

    /// <summary>
    /// foamliu, 2009/04/27, 规则引擎的随机问题生成器, 主要用于压力/性能测试.
    /// </summary>
    public class ProblemGen
    {
        // 每条规则条件式最大数.
        public const int MaxConditions = 1;   // Max condition number for a rule (both lhs and rhs)

        // 实体个数
        public const int IndNum = 10000;
        // 谓词个数
        public const int PredNum = 10000;

        // 规则个数
        private int m_ruleNum;
        // 事实个数
        private int m_factNum;
        // 生成的问题保存的文件
        private string m_fileName;
        // 随机数生成器
        private Random m_rand;

        //public Collection<Production> Productions = new Collection<Production>();
        //public Collection<WME> Facts = new Collection<WME>();

        /// <summary>
        /// foamliu, 2009/04/27, 构造器.
        /// </summary>
        /// <param name="fileName">生成的问题保存在那</param>
        /// <param name="ruleNum">生成随机规则数</param>
        /// <param name="factNum">生成随机事实数</param>
        public ProblemGen(string fileName, int ruleNum, int factNum)
        {
            this.m_fileName = fileName;
            this.m_ruleNum = ruleNum;
            this.m_factNum = factNum;

            this.m_rand = new Random();
        }

        /// <summary>
        /// 得到一个随机个体
        /// </summary>
        /// <returns></returns>
        private string GetRandInd()
        {
            return "ind" + Math.Floor(m_rand.NextDouble() * IndNum);
        }

        /// <summary>
        /// 得到一个随机谓词
        /// </summary>
        /// <returns></returns>
        private string GetRandPred()
        {
            return "^pred" + Math.Floor(m_rand.NextDouble() * IndNum);
        }

        /// <summary>
        /// foamliu, 2009/04/27, 生成规则.
        /// </summary>
        /// <param name="filestream"></param>
        /// <param name="bf">二进制序列化</param>
        private void GenerateRules(FileStream filestream, BinaryFormatter bf)
        {
            Logger.Info("Started generating {0} rules...", m_ruleNum);
            CommandLineProgressBar progress = new CommandLineProgressBar(m_ruleNum);

            for (int i = 0; i < m_ruleNum; i++)
            {
                Production prod = new Production();
                int numLhs = (int)Math.Floor(m_rand.NextDouble() * MaxConditions);
                int numRhs = (int)Math.Floor(m_rand.NextDouble() * MaxConditions);
                for (int j = 0; j < numLhs; j++)
                    prod.LHS.Add(new PositiveCondition(GetRandInd(), GetRandPred(), GetRandInd()));
                for (int j = 0; j < numRhs; j++)
                    prod.RHS.Add(new AssertCondition(GetRandInd(), GetRandPred(), GetRandInd()));
                //Productions.Add(prod);
                progress.Draw(i + 1);

                bf.Serialize(filestream, prod);
            }
        }

        /// <summary>
        /// foamliu, 2009/04/27, 生成事实.
        /// </summary>
        /// <param name="filestream"></param>
        /// <param name="bf">二进制序列化</param>
        private void GenerateFacts(FileStream filestream, BinaryFormatter bf)
        {
            Logger.Info("Started generating {0} facts...", m_factNum);
            CommandLineProgressBar progress = new CommandLineProgressBar(m_factNum);

            for (int i = 0; i < m_factNum; i++)
            {
                WME wme = new WME(GetRandInd(), GetRandPred(), GetRandInd());
                //Facts.Add(wme);
                progress.Draw(i + 1);

                bf.Serialize(filestream, wme);
            }
        }

        /// <summary>
        /// foamliu, 2009/04/27, 开始生成.
        /// </summary>
        public void Generate()
        {
            FileStream filestream = new FileStream(m_fileName, FileMode.Create);
            BinaryFormatter bf = new BinaryFormatter();

            bf.Serialize(filestream, m_ruleNum);
            bf.Serialize(filestream, m_factNum);

            GenerateRules(filestream, bf);
            GenerateFacts(filestream, bf);

            filestream.Close();
        }
    }
}
